#include <SPI.h>
#include "CC2500.h"
#include "MJX_v1.h"

// NOTE: ATMega2560
#define CSN_PIN 48
#define SCK_PIN 52
#define MOSI_PIN 51
#define MISO_PIN 50

// Pins for AATMega2560:
//  48 = CSN, 50 = MISO, 51 = MOSI, 52 = SCK

#define USE_TIMER_INTERRUPTS

// in microseconds
// MJX V1 sends packets every 8ms (-> 125 packets/sec)
#define TIMER_INTERVAL 8000

#ifdef USE_TIMER_INTERRUPTS
#include <avr/io.h>
#include <avr/interrupt.h>
#endif

CC2500 radio(CSN_PIN, SCK_PIN, MOSI_PIN, MISO_PIN);
MJX_TX tx(radio);

uint8_t throttle, flags;
int8_t yaw, pitch, roll;

int a0, a1, a2, a3;
int a0min, a0max;
int a1min, a1max;
int a2min, a2max;
int a3min, a3max;

void calibrate()
{
  a0min = 300; a0max=700;  // throttle
  a1min = 300; a1max=700;
  a2min = 300; a2max=700;
  a3min = 300; a3max=700;

  long a = analogRead(A0);
  Serial.print(" A0: ");
  Serial.print(a);
  a = analogRead(A1);
  Serial.print(" A1: ");
  Serial.print(a);
  a = analogRead(A2);
  Serial.print(" A2: ");
  Serial.print(a);
  a = analogRead(A3);
  Serial.print(" A3: ");
  Serial.print(a);
  Serial.print("\n");

}

void initInput()
{
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  calibrate();
}

bool readInput()
{
  bool changed = false;
  long a;
  a = analogRead(A0);
  if (a < a0min) a0min = a;
  if (a > a0max) a0max = a;
  a = (a-a0min)*255/(a0max-a0min);
  if (a != a0) { changed = true; a0 = a; }

  a = analogRead(A1);
  if (a < a1min) a1min = a;
  if (a > a1max) a1max = a;
  a = (a-a1min)*255/(a1max-a1min);
  if (a != a1) { changed = true; a1 = a; }

  a = analogRead(A2);
  if (a < a2min) a2min = a;
  if (a > a2max) a2max = a;
  a = (a2max-a)*255/(a2max-a2min);
  if (a != a2) { changed = true; a2 = a; }

  a = analogRead(A3);
  if (a < a3min) a3min = a;
  if (a > a3max) a3max = a;
  a = (a3max-a)*255/(a3max-a3min);
  if (a != a3) { changed = true; a3 = a; }
  return changed;
}


int counter = 0;
int direction = 1;
int lastmillis = 0;
bool bind = true;
bool startbind = false;
bool calibrated = false;


void setup() 
{
  Serial.begin(115200);
  Serial.print("Starting up...\n");

  initInput();
  readInput();

  throttle = 0; yaw = 0; pitch = 0; roll = 0; flags = 0;
  counter = 0;
  direction = 1;
  bind = true;
  startbind = false;

  uint8_t txid[] = {0,0,0};
  tx.setTXId(txid);  
  tx.begin();

  uint8_t partnum = radio.sendCommand(CC2500_OFF_READ_BURST | CC2500_REG_PARTNUM, 0x00);
  uint8_t version = radio.sendCommand(CC2500_OFF_READ_BURST | CC2500_REG_VERSION, 0x00);
  uint8_t marcstate = radio.sendCommand(CC2500_OFF_READ_BURST | CC2500_REG_MARCSTATE, 0x00);
  
  Serial.print(" Part: ");
  Serial.print(partnum, HEX);
  Serial.print(" version: ");
  Serial.print(version, HEX);
  Serial.print(" state: ");
  Serial.print(marcstate, HEX);
  Serial.print("\n");



#ifdef USE_TIMER_INTERRUPTS
#define RESOLUTION 65536
  unsigned char clockSelectBits;
  long microseconds = TIMER_INTERVAL;
  // That's simply copied from "TimerOne" library
  TCCR1A = 0;
  TCCR1B = _BV(WGM13);
  long cycles = (F_CPU * microseconds) / 2000000;                                // the counter runs backwards after TOP, interrupt is at BOTTOM so divide microseconds by 2
  if(cycles < RESOLUTION)              clockSelectBits = _BV(CS10);              // no prescale, full xtal
  else if((cycles >>= 3) < RESOLUTION) clockSelectBits = _BV(CS11);              // prescale by /8
  else if((cycles >>= 3) < RESOLUTION) clockSelectBits = _BV(CS11) | _BV(CS10);  // prescale by /64
  else if((cycles >>= 2) < RESOLUTION) clockSelectBits = _BV(CS12);              // prescale by /256
  else if((cycles >>= 2) < RESOLUTION) clockSelectBits = _BV(CS12) | _BV(CS10);  // prescale by /1024
  else        cycles = RESOLUTION - 1, clockSelectBits = _BV(CS12) | _BV(CS10);  // request was out of bounds, set as maximum
  ICR1 = cycles;                                                                 // ICR1 is TOP in p & f correct pwm mode
  TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));
  TIMSK1 = _BV(TOIE1);                                     // sets the timer overflow interrupt enable bit
  sei();                                                   // ensures that interrupts are globally enabled  
  TCCR1B |= clockSelectBits;                                                     // reset clock select register (-> starts timer)
#endif
} 



void loop() 
{
#ifdef USE_TIMER_INTERRUPTS
}

// interrupt service routine
ISR(TIMER1_OVF_vect)
{
#endif
  
  bool changed = readInput();
  if (bind) {
    a0 = a1 = a2 = a3 = 0; // zero channels on bind
    throttle = a0;
    flags |= 0xc0;
    
    if (startbind ) {
//        Serial.write("Starting bind...\n");
    }
  } else {
    // Normal operation
    flags &= 0x3f;
    throttle = a0;
    yaw = a1 < 0x80 ? 0x7f - a1 : a1;
    roll = a2 < 0x80 ? 0x7f - a2 : a2 ;
    pitch = a3 < 0x80 ? 0x7f - a3 : a3;
  }

  // TESTING SAFETY: Limit throttle to small value
  //if (throttle > 16) throttle = 16;
  
  if (startbind) tx.command(throttle, yaw, pitch, roll, flags);
  
  
  if (Serial.available()) {
    char c = toupper(Serial.read());
    if (c == 'B') {
      Serial.print("Trigger Bind\n");
      startbind=true;
      bind=true;
    }
    if (c == 'T') {
      Serial.print("Starting normal mode\n");
      bind=false;
    }
    if (c == 'R') {
      Serial.print("Reset\n");
      startbind=false;
      bind=true;
    }
    // 4 (0x10) = cam
    if ((c >= '0')&&(c <= '7')) {
      int bn = c-'0';
      if (flags & (1 << bn))
        flags &= 0xff-(1 << bn);
      else
        flags |= (1 << bn);
      Serial.print("Flags now ");Serial.print(flags,HEX); Serial.print("\n");
    }
  }
  
  // 8 ms between packets ...
#ifndef USE_TIMER_INTERRUPTS
  delayMicroseconds(8000);
#endif
}

